'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var utils = require('./utils.js');
var stringResources = require('./stringResources.js');

var defaultOptions = {};
function Accessibility(options) {
  var controller;
  var pageInitialized = false;
  var areas;
  var lastArea;
  var keyMap = {
    //keys for navigating on areas. Used in conjunction with CTRL+ALT to avoid duplicating default shortcuts behavior.
    CONFIRM_KEY: 13,
    CONTENT_AREA_KEY: 67,
    //C
    DOCUMENT_MAP_AREA_KEY: 68,
    //D
    MENU_AREA_KEY: 77,
    //M
    PARAMETERS_AREA_KEY: 80
    //P
  };
  options = $.extend({}, defaultOptions, options);
  controller = options.controller;
  if (!controller) {
    throw "No controller (telerikReporting.ReportViewerController) has been specified.";
  }
  controller.reportLoadComplete(function() {
    setAccessibilityUI();
    var content = findContentArea();
    if (content.length > 0) {
      content.focus();
    }
  }).pageReady(function(event, page) {
    initPage(page);
    pageInitialized = true;
  }).error(function(e, message) {
    focusOnErrorMessage();
    window.setTimeout(setAccessibilityUI, 500);
  });
  controller.updateUI(function(e) {
    if (pageInitialized) {
      setPageSelector();
      decorateMenuItems();
    }
  });
  function setAccessibilityUI() {
    if (!areas) {
      initAreas();
      $(document.body).off("keydown", processKeyDown);
      $(document.body).on("keydown", processKeyDown);
    }
  }
  function focusOnErrorMessage() {
    var selectorChain = ["div.trv-pages-area", "div.trv-error-message"];
    var $errMsg = utils.findElement(selectorChain);
    if ($errMsg.length === 0) {
      return;
    }
    $errMsg.attr("tabIndex", 0);
    $errMsg.focus();
  }
  function initPage(page) {
    if (!page) {
      return;
    }
    setAccessibilityUI();
    var area = areas[keyMap.CONTENT_AREA_KEY];
    setContentAreaKeyDown(area);
  }
  function setPageSelector() {
    var $pagers = $(".trv-report-pager");
    if ($pagers.length > 0) {
      var pageNumber = controller.getCurrentPageNumber();
      var pageCount = controller.getPageCount();
      utils.each($pagers, function() {
        var $pager = $(this);
        $pager.attr("aria-label", utils.stringFormat(stringResources.stringResources.ariaLabelPageNumberSelector, [pageNumber, pageCount]));
        var $pageInputs = $pager.find("input[data-role=telerik_ReportViewer_PageNumberInput]");
        if ($pageInputs.length > 0) {
          utils.each($pageInputs, function() {
            var $this = $(this);
            $this.attr("aria-label", stringResources.stringResources.ariaLabelPageNumberEditor);
            $this.attr("min", "1");
            $this.attr("max", "" + pageCount);
          });
        }
      });
    }
  }
  function initAreas() {
    areas = {};
    areas[keyMap.DOCUMENT_MAP_AREA_KEY] = findDocumentMapArea();
    areas[keyMap.MENU_AREA_KEY] = findMenuArea();
    areas[keyMap.CONTENT_AREA_KEY] = findContentArea();
    var parametersArea = findParametersArea();
    if (parametersArea) {
      areas[keyMap.PARAMETERS_AREA_KEY] = parametersArea;
      setParameterEditorsKeyDown(parametersArea);
    }
  }
  function findContentArea() {
    return utils.findElement(["div[data-role=telerik_ReportViewer_PagesArea]"]);
  }
  function findDocumentMapArea() {
    return utils.findElement(["div[data-role=telerik_ReportViewer_DocumentMapArea]", "div[data-role=treeview]"]);
  }
  function findMenuArea() {
    return utils.findElement("ul[data-role=telerik_ReportViewer_MainMenu]");
  }
  function findParametersArea() {
    return utils.findElement(["div[data-role=telerik_ReportViewer_ParametersArea]", "div.trv-parameters-area-content"]);
  }
  function processKeyDown(event) {
    if (!areas) {
      return;
    }
    if (!(event.altKey && event.ctrlKey)) {
      return;
    }
    var currentArea = areas[event.which];
    if (!currentArea) {
      return;
    }
    if (!IsAreaContainerVisible(currentArea.parent())) {
      return;
    }
    var className = "k-focus";
    if (lastArea) {
      lastArea.removeClass(className);
    }
    currentArea.addClass(className);
    currentArea.focus();
    lastArea = currentArea;
    event.preventDefault();
  }
  function setParameterEditorsKeyDown(parametersAreaContent) {
    if (parametersAreaContent.length === 0) {
      return;
    }
    var $paramsArea = parametersAreaContent.parent("div[data-role=telerik_ReportViewer_ParametersArea]");
    if (!IsAreaContainerVisible($paramsArea)) {
      return;
    }
    utils.each(parametersAreaContent.children(), function() {
      $(this).keydown(function(event) {
        if (event.which == keyMap.CONFIRM_KEY) {
          var paramsButton = $paramsArea.find("button.trv-parameters-area-preview-button");
          paramsButton.focus();
          event.preventDefault();
        }
      });
    });
  }
  function IsAreaContainerVisible(container) {
    return container && !(container.hasClass("k-collapsed") || container.hasClass("trv-hidden"));
  }
  function setContentAreaKeyDown(contentArea) {
    if (!contentArea) {
      return;
    }
    var actions = contentArea.find("div [data-reporting-action]");
    if (!actions.length > 0) {
      return;
    }
    utils.each(actions, function() {
      var $action = $(this);
      $action.keydown(function(event) {
        if (event.which == keyMap.CONFIRM_KEY) {
          $action.click();
        }
      });
    });
  }
  function decorateMenuItems() {
    var menuAreas = areas[keyMap.MENU_AREA_KEY];
    if (!menuAreas) {
      return;
    }
    utils.each(menuAreas, function() {
      var $menu = $(this);
      var menuItems = $menu.children("li.k-item");
      utils.each(menuItems, function() {
        var $menuItem = $(this);
        if (!$menuItem.hasClass("trv-report-pager")) {
          var ariaLabel = $menuItem.attr("aria-label");
          var expandableSr = utils.stringFormat(". {0}", [stringResources.stringResources.ariaLabelExpandable]);
          var expandable = $menuItem.find("ul").length > 0 && ariaLabel.indexOf(expandableSr) < 0 ? expandableSr : "";
          var selectedSr = utils.stringFormat(". {0}", [stringResources.stringResources.ariaLabelSelected]);
          var selected = $menuItem.hasClass("k-selected") && ariaLabel.indexOf(selectedSr) < 0 ? selectedSr : "";
          var label = ariaLabel + expandable + selected;
          $menuItem.attr("aria-label", label);
          if ($menuItem.hasClass("k-disabled")) {
            $menuItem.attr("aria-disabled", "true");
          } else {
            $menuItem.removeAttr("aria-disabled");
          }
        }
      });
    });
  }
  function setKeyMap(keyMapValues) {
    keyMap = keyMapValues;
    areas = void 0;
  }
  function getKeyMap() {
    return keyMap;
  }
  return {
    getKeyMap,
    setKeyMap
  };
}

exports.Accessibility = Accessibility;
