import { extend, rTrim, $ajax, stringFormat } from './utils.js';

var JSON_CONTENT_TYPE = "application/json; charset=UTF-8";
var URLENCODED_CONTENT_TYPE = "application/x-www-form-urlencoded; charset=UTF-8";
var HTTP_GET = "GET";
var HTTP_POST = "POST";
var HTTP_DELETE = "DELETE";
var defaultOptions = {};
function ServiceClient(options) {
  options = extend({}, defaultOptions, options);
  var baseUrl = rTrim(options.serviceUrl || options.baseUrl, "\\/");
  var loginPromise;
  var _ajax = $ajax;
  function validateClientID(clientID) {
    if (!clientID)
      throw "Invalid clientID";
  }
  function urlFromTemplate(template, args) {
    args = extend({}, { baseUrl }, args);
    return stringFormat(template, args);
  }
  function getHeaderSettings(authorizationToken) {
    return authorizationToken ? {
      headers: {
        Authorization: "Bearer " + authorizationToken
      }
    } : {};
  }
  function login() {
    if (!loginPromise) {
      var loginInfo = options.loginInfo;
      if (loginInfo && loginInfo.url && (loginInfo.username || loginInfo.password)) {
        loginPromise = _ajax({
          url: loginInfo.url,
          type: HTTP_POST,
          data: {
            grant_type: "password",
            username: loginInfo.username,
            password: loginInfo.password
          },
          dataType: "json",
          contentType: URLENCODED_CONTENT_TYPE
        }).then(function(result) {
          return result.access_token;
        });
      } else {
        loginPromise = Promise.resolve();
      }
    }
    return loginPromise;
  }
  return {
    _urlFromTemplate: urlFromTemplate,
    registerClient: function(settings) {
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate("{baseUrl}/clients"),
            dataType: "json",
            //Used to avoid Chrome caching functionality for simple async requests
            //when the first request is not completed the second request is not send
            //and the same response is used for both of the request. In this case the
            //second client is not registered to the service and the same clientId is used.
            data: JSON.stringify({ timeStamp: Date.now() })
          }
        );
        return _ajax(ajaxSettings);
      }).then(function(clientData) {
        if (clientData.Message) {
          throw clientData.Message;
        }
        return clientData.clientId;
      });
    },
    unregisterClient: function(clientID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_DELETE,
            url: urlFromTemplate("{baseUrl}/clients/{clientID}", { clientID })
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getParameters: function(clientID, report, parameterValues, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate("{baseUrl}/clients/{clientID}/parameters", { clientID }),
            contentType: JSON_CONTENT_TYPE,
            dataType: "json",
            data: JSON.stringify({ report, parameterValues })
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    createReportInstance: function(clientID, report, parameterValues, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate("{baseUrl}/clients/{clientID}/instances", { clientID }),
            contentType: JSON_CONTENT_TYPE,
            dataType: "json",
            data: JSON.stringify({ report, parameterValues })
          }
        );
        return _ajax(ajaxSettings);
      }).then(function(instanceData) {
        return instanceData.instanceId;
      });
    },
    deleteReportInstance: function(clientID, instanceID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_DELETE,
            url: urlFromTemplate("{baseUrl}/clients/{clientID}/instances/{instanceID}", { clientID, instanceID })
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    createReportDocument: function(clientID, instanceID, format, deviceInfo, useCache, baseDocumentID, actionID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        deviceInfo = deviceInfo || {};
        deviceInfo["BasePath"] = baseUrl;
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate("{baseUrl}/clients/{clientID}/instances/{instanceID}/documents", { clientID, instanceID }),
            contentType: JSON_CONTENT_TYPE,
            dataType: "json",
            data: JSON.stringify(
              {
                format,
                deviceInfo,
                useCache,
                baseDocumentID,
                actionID
              }
            )
          }
        );
        return _ajax(ajaxSettings);
      }).then(function(documentData) {
        return documentData.documentId;
      });
    },
    sendDocument: function(clientID, instanceID, documentID, mailArgs, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate(
              "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}/send",
              { clientID, instanceID, documentID }
            ),
            contentType: JSON_CONTENT_TYPE,
            data: JSON.stringify(
              {
                from: mailArgs.from,
                to: mailArgs.to,
                cc: mailArgs.cc,
                subject: mailArgs.subject,
                body: mailArgs.body
              }
            )
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    deleteReportDocument: function(clientID, instanceID, documentID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_DELETE,
            url: urlFromTemplate(
              "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}",
              { clientID, instanceID, documentID }
            )
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getDocumentInfo: function(clientID, instanceID, documentID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate(
              "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}/info",
              {
                clientID,
                instanceID,
                documentID
              }
            ),
            dataType: "json"
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getPage: function(clientID, instanceID, documentID, pageNumber, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate(
              "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}/pages/{pageNumber}",
              {
                clientID,
                instanceID,
                documentID,
                pageNumber
              }
            ),
            dataType: "json"
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    get: function(url) {
      var ajaxSettings = {
        type: HTTP_GET,
        url
      };
      return _ajax(ajaxSettings);
    },
    formatDocumentUrl: function(clientID, instanceID, documentID, queryString) {
      var url = urlFromTemplate(
        "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}",
        {
          clientID,
          instanceID,
          documentID
        }
      );
      if (queryString) {
        url += "?" + queryString;
      }
      return url;
    },
    getDocumentFormats: function(settings) {
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate("{baseUrl}/formats"),
            // anonymous
            dataType: "json"
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getServiceVersion: function(settings) {
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate("{baseUrl}/version"),
            // anonymous
            dataType: "json"
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getResource: function(clientID, instanceID, documentID, resourceID, settings) {
      validateClientID(clientID);
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate(
              "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}/resources/{resourceID}",
              {
                clientID,
                instanceID,
                documentID,
                resourceID
              }
            ),
            dataType: (settings == null ? void 0 : settings.dataType) || "json"
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getSearchResults: function(clientID, instanceID, documentID, searchToken, matchCase, matchWholeWord, useRegex, settings) {
      validateClientID(clientID);
      var searchUrl = urlFromTemplate(
        "{baseUrl}/clients/{clientID}/instances/{instanceID}/documents/{documentID}/search",
        {
          clientID,
          instanceID,
          documentID
        }
      );
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: searchUrl,
            contentType: JSON_CONTENT_TYPE,
            dataType: "json",
            data: JSON.stringify(
              {
                searchToken,
                matchCase,
                matchWholeWord,
                useRegularExpressions: useRegex
              }
            )
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    setAccessToken: function(accessToken) {
      loginPromise = Promise.resolve(accessToken);
    },
    login,
    keepClientAlive: function(clientID, settings) {
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_POST,
            url: urlFromTemplate("{baseUrl}/clients/keepAlive/{clientID}", { clientID })
          }
        );
        return _ajax(ajaxSettings);
      });
    },
    getClientsSessionTimeoutSeconds: function(settings) {
      return login().then(function(authorizationToken) {
        var ajaxSettings = extend(
          getHeaderSettings(authorizationToken),
          settings,
          {
            type: HTTP_GET,
            url: urlFromTemplate("{baseUrl}/clients/sessionTimeout")
          }
        );
        return _ajax(ajaxSettings);
      }).then(function(sessionTimeoutData) {
        return sessionTimeoutData.clientSessionTimeout;
      });
    }
  };
}

export { ServiceClient };
